#!/usr/bin/perl -w

# Program include.pl

# (c) Gary C. Kessler, 2009 [gck@garykessler.net]


# Ok, one of my pet peeves about HTML is that there's no way to include
# text from another file unless you have fancy tools or some server support.
# This is particularly important if you want to use some standard headers or
# navigation bar or other frame-like features but don't want to use frames.
# This is also important for those of us who still write our HTML by hand!
# =(sigh)=

# See include_readme.txt for usage instructions.

# ********************************************************
# MAIN PROGRAM BLOCK
# ********************************************************

# Initialize global variables

$version = "2.2";
$build_date = "2 February 2011";

# $source is the name of the file to modify. A backup copy is created
# with ".bak" appended to the filename.

$source = "";

# Counters ...

$numlinesin = 0;          # Number of lines read from INFILE
$numlinesout = 0;         # Number of original lines written to OUTFILE
$numblocksinserted = 0;   # Number of INCLUDE blocks inserted
$numlinesinserted = 0;   # Number of INCLUDE lines inserted

# $copy tells the program whether to copy files from the main INFILE to
# OUTFILE. Under ordinary circumstances, $copy = 1 (true). It is set to
# false (0) only to skip the existing lines within the main INFILE's
# INCLUDE block.

$copy = 1;

print "HTML Include V$version - Gary C. Kessler ($build_date)\n\n";

# Parse command line switches and request missing information.
# If user has supplied the -h, -v, or an illegal switch, the subroutine
# returns a value of 1 and the program stops.

if (parsecommandline ())
  { exit 1; };

# Rename the source file to become the backup file, which then becomes
# the input file for this program.

$backup = $source.".bak";
rename $source, $backup;

print "\nSource = $source \n";
print "\nBackup = $backup \n";

# Open input ($backup) and output ($source) files and stop program if error

open (INFILE, "<", $backup)
  or die "Can't open input file \"$backup\": $!";

open (OUTFILE, ">", $source)
  or die "Can't open output file \"$source\": $!";

# Read a line from INFILE. There are three possible cases:
#
# 1) If this is the start of an INCLUDE block, write the line to OUTFILE,
#    set $copy to false, and transfer control to a subroutine that reads
#    the file to be INCLUDEd and writes it to the OUTFILE.
# 2) If this is the end of an INCLUDE block, write the line to OUTFILE and
#    set $copy to true.
# 3) For all other lines, write the line to OUTFILE if $copy = true. (I.e.,
#    write the line to OUTFILE unless it is inside the INFILE's existing
#    INCLUDE block, which will be overwritten with the new INCLUDE block).

while (not (eof INFILE))
  {
  $line = <INFILE>;
  $numlinesin++;

  if ($line =~ /^<!--#INCLUDE/)
    {
    print OUTFILE $line;

    $numlinesout++;
    ($d1, $includefile, $d3) = ($line =~ /(\S+) (\S+\.\S+) (\S+)/);
    $d2 = $d1; $d2 = $d3; # Supress error message
    if (include_file ())
      { exit 1} ;
    $copy = 0;
    }
  elsif ($line =~ /^<!--#EOI/)
    {
    print OUTFILE $line;
    $numlinesout++;
    $copy = 1;
    }
  elsif ($copy)
    {
    print OUTFILE $line;
    $numlinesout++;
    } ;
  } ;

close INFILE;
close OUTFILE;

# Finish up!

print "Input file: $backup\n";
print "Output file: $source\n";
print "Number of lines read from $source: $numlinesin\n";
print "Number of lines read from original $source and then written to new file: $numlinesout\n";
print "Number of INCLUDE blocks read: $numblocksinserted\n";
print "Number of INCLUDE lines inserted: $numlinesinserted\n\n";
print "Cheers!\n\n";


# ********************************************************
# *****           SUBROUTINE BLOCKS                  *****
# ********************************************************


# ********************************************************
# include_file
# ********************************************************

# When the #INCLUDE directive is found, control comes to this routine
# in order to open the INCLUDE file and write it to the OUTFILE

sub include_file
{
my $state = 0;

# Open the INCLUDEFILE if available

open (INCLUDEFILE, "<", $includefile)
  or die "Can't open include file \"$includefile\": $!";

# Read the INCLUDEFILE and copy to OUTFILE

$numblocksinserted++;
while (not (eof INCLUDEFILE))
  {
  $newline = <INCLUDEFILE>;
  $numlinesinserted++;
  print OUTFILE $newline;
  } ;

close INCLUDEFILE;

return $state;
}


# ********************************************************
# parsecommandline
# ********************************************************

# Parse command line for any switches that are present. Query
# user for any missing information

# Return $state = 1 to indicate that the program should stop
# immediately (-h, -v, or unrecognized switch)

sub parsecommandline
{
my $state = 0;

# Parse command line switches ($ARGV array of length $#ARGV)

if ($#ARGV >= 0)
  { 
  for ($i = 0; $i <= $#ARGV; $i++)
    {
    PARMS:
      {
      $ARGV[$i] eq "-f" && do
         {
         $source = $ARGV[$i+1];
         $i++;
         last PARMS;
         };
      $ARGV[$i] eq "-v" && do
         {
         $state = 1;
         return $state;
         };
      $ARGV[$i] eq "-h" && do
         {
         help_text();
         $state = 1;
         return $state;
         };
      do
         {
         print "Invalid parameter \"$ARGV[$i]\".\n\n";
         print "Usage: include [-f input_file]\n";
         print "       include {[-h] | [-v]}\n\n";
         $state = 1;
         return $state;
         };
      };
    };
  };

# Prompt for file names, if not supplied

if ($source eq "")
  {
  print "\nEnter the input file name: ";
  chomp ($source = <STDIN>);
  };

return $state;
}

# ********************************************************
# help_text
# ********************************************************

# Display the help file

sub help_text
{
print "Program usage: include [-f input_file]\n";
print "               include {[-h] | [-v]}\n\n";
print " where: -f is the input file name\n";
print "        -h prints this help file\n";
print "        -v displays the program version number\n\n";
print "If no switch is provided, the program will ask for the\n";
print "input file name.\n\n";
print "The original input file is automatically renamed to have a\n";
print "'.bak' extension, and output is directed to a file using the\n";
print "original input file name. Read 'include_readme.txt' for a\n";
print "full description of the use of the #INCLUDE directive and\n";
print "how to set up a batch file to change several files at once.\n";
return;
}

